%%% GDP SOLERA %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Clear memory
clear
clc
close all

% Set tasks
re_estimate = true;

% Set filename
fname = 'v0';

% Add paths and directories
addpath(['..' filesep 'simulation' filesep 'v0' filesep 'functions'])
data_dir = ['..' filesep 'data' filesep 'output' filesep];
fig_dir  = ['output' filesep 'figures' filesep];
mat_dir  = ['output' filesep 'matfiles' filesep];
tab_dir  = ['output' filesep 'tables' filesep];

% Set code parameters
rng(1918)
settings           = struct();
n_MC               = 2e4;
settings.n_MC      = n_MC;
settings.n_burn    = n_MC;
settings.n_thin    = 5;
settings.plot_MCMC = true;
settings.plot_each = 200;

% Load and arrange data
datasets  = load([data_dir 'data.mat']);
time      = datetime(datenum(datasets.GDE.time, 'yyyyQQ'), 'convertfrom', 'datenum');
subsample = (datetime(1984, 1, 1) <= time) & (time < datetime(2026, 1, 1));
time      = time(subsample, :); 
Y_GDE     = 100*log( [datasets.GDE.early(subsample, :), ...
                      datasets.GDE.late(subsample, :), ...
                      datasets.GDE.compr(subsample, :)] )';
Y_GDI     = 100*log( [datasets.GDI.early(subsample, :), ...
                      datasets.GDI.late(subsample, :), ...
                      datasets.GDI.compr(subsample, :)] )';
Y         = [Y_GDE; Y_GDI];

% Compute an store dimensions in a vector
N_c  = length(datasets.GDE.compr_aux);
N_E  = size(Y_GDE, 1);
N_I  = size(Y_GDI, 1);
T    = nnz(subsample);
dims = [N_c, N_E, N_I, T];
                   
% Define vintage indicators    
date_vintage = vintage2date( [datasets.GDE.early_vintage(subsample, :), ...
                              datasets.GDE.late_vintage(subsample, :), ...
                              datasets.GDE.compr_vintage(subsample, :), ...
                              datasets.GDI.early_vintage(subsample, :), ...
                              datasets.GDI.late_vintage(subsample, :), ...
                              datasets.GDI.compr_vintage(subsample, :)]' );

% Define signal indicators
delta_signal = NaN(N_E+N_I, T);
date_signal  = vintage2date( datasets.GDE.compr_aux );
for i_c = 1:(N_c-1)
    delta_signal(date_vintage >= date_signal(i_c) & date_vintage < date_signal(i_c+1)) = i_c;
end
delta_signal(date_vintage >= date_signal(N_c)) = N_c;
delta_signal = fillmissing(delta_signal, 'next', 2);     % impute missing entries of old signals using next available
delta_signal = fillmissing(delta_signal, 'previous', 2); % impute missing entries of new/discontinued signals using previous available

% Store indicators in 3D array
delta = zeros(N_E+N_I, N_c, T);
for i_c = 1:N_c
    delta(:, i_c, :) = double(delta_signal == i_c);
end

% Construct data with available measurements
ref_vintage = time(end) + calmonths(12);
mat_vintage = (date_vintage <= ref_vintage);
Y_vintage   = NaN(N_E+N_I, T);
Y_vintage(mat_vintage) = Y(mat_vintage);
Y_filtering = Y_vintage;

% Avoid pandemic quarters for estimation
Y_vintage(:, (year(time) == 2020 & quarter(time) == 2)) = NaN;
Y_vintage(:, (year(time) == 2020 & quarter(time) == 3)) = NaN;

% Inputs for plots and tables
figsize    = {'units', 'normalized', 'position', [0 0 1 1]};
ax_fonts   = {'fontname', 'times', 'fontsize', 10, 'linewidth', 0.5};
black      = [  0,   0,   0];
blue       = [ 51,  51, 153]/255;
red        = [153,  51,  51]/255;
green      = [ 51, 153,  51]/255;
dark_green = [  0, 100,   0]/255;


%% PLOTS OF STATISTICAL DISCREPANCIES

% Set names and selection indicators
S_name      = {'GDE Advance-Third', 'GDE Second-Third', 'GDI Second-Third', ...
               'GDE-GDI Second', 'GDE-GDI Third', 'GDE-GDI Annual 1', 'GDE-GDI Annual 2',  'GDE-GDI Annual 3', ...
               'GDE-GDI Comprehensive 1', 'GDE-GDI Comprehensive 2', 'GDE-GDI Comprehensive 3', 'GDE-GDI Comprehensive 4', 'GDE-GDI Comprehensive 5'};
S_plus      = [1, 2, N_E+1, 2,     3,     4,     5,     6,     7,     8,     9,     10,    11];
S_minus     = [3, 3, N_E+2, N_E+1, N_E+2, N_E+3, N_E+4, N_E+5, N_E+6, N_E+7, N_E+8, N_E+9, N_E+10];
n_S         = length(S_plus);
group_name  = {'Early revisions', 'Early and latest discrepancies', 'Comprehensive discrepancies'};
group_fname = {'revisions', 'noncompr', 'compr'};
group_color = {{blue; red; green}, {blue; red; black; green; dark_green}, {blue; red; black; green; dark_green}};
S_group     = {1:3, 4:8, 9:13};
n_group     = length(S_group);

% Define function handle for autocovariance calculation
auto_cov = @(x, lag) mean((x(:, (lag+1):end) - mean(x(:, (lag+1):end), 2, 'omitnan')) ...
                      .* (x(:, 1:(end-lag)) - mean(x(:, 1:(end-lag)), 2, 'omitnan')), 2, 'omitnan');

% Compute discrepancy in the data
S_data = NaN(n_S, T);
for i_S = 1:n_S, S_data(i_S, :) = Y_vintage(S_plus(i_S), :) - Y_vintage(S_minus(i_S), :); end
S_data(S_group{1}, (quarter(time) == 3 & year(time) == 2003)) = NaN;
                  
% Compute autocovariance in the data
n_lag       = 12;
ACovF_data  = NaN(n_S, n_lag+1);
ACorrF_data = NaN(n_S, n_lag+1);
for i_lag = 0:n_lag
    for i_S = 1:n_S 
        ACovF_data(i_S, i_lag+1)  = auto_cov(S_data(i_S, :), i_lag);
        ACorrF_data(i_S, i_lag+1) = ACovF_data(i_S, i_lag+1)/auto_cov(S_data(i_S, :), 0);
    end
end

% Plot statistical discrepancies
for i_group = 1:n_group
    
    % Create figure with statistical discrepancies
    fig0  = figure();
    ax0   = axes();
    plot0 = plot(ax0, time, S_data(S_group{i_group}, :)');
    set(plot0, 'linewidth', 2)
    set(plot0, 'linestyle', '-')
    set(plot0, {'color'}, group_color{i_group})
    
    % Add extras
    t_min = min(time(~all(isnan(S_data(S_group{i_group}, :)), 1)));
    t_max = max(time(~all(isnan(S_data(S_group{i_group}, :)), 1)));
    xlim(ax0, [t_min, t_max])
    ylabel(ax0, group_name{i_group})
    line(ax0, [t_min, t_max], [0, 0], 'linewidth', 0.5, 'linestyle', '--', 'color', [0, 0, 0])
    legend(ax0, S_name(S_group{i_group}), 'location', 'best', 'orientation', 'vertical')
    
    % Tune ax handle
    set(ax0, ax_fonts{:})
    set(ax0, 'box', 'on')
    set(ax0, 'linewidth', 0.5)
    
    % Save and close figure
    print(fig0, [fig_dir sprintf('discrepancies_data_%s.eps', group_fname{i_group})], '-depsc')
    
end


%% GIBBS SAMPLING (V0: DIAGONAL VAR + UNRESTRICTED COVARIANCES - BASELINE)

close all

% Set prior
prior    = construct_prior(dims);
mu       = mean(Y(:, 1), 'omitnan')*ones(N_c, 1);
prior.mu = mu;

if (re_estimate == true) || (~exist([mat_dir sprintf('estimates_%s.mat', fname)], 'file'))
% Set random number generator seed
rng(1914)

% Run Gibbs sampler
[param_Gibbs, states, ML] = Gibbs_sampler(Y_vintage, delta, prior, settings);

% Carry signal extraction and data simulation
Y_sim = NaN(N_E+N_I, T, n_MC);
for i_MC = 1:n_MC
    if isfield(prior, 'mu'), mu = prior.mu; else, mu = mean(Y(:, 1), 'omitnan')*ones(N_c, 1); end
    SSM                = construct_SSM(param_Gibbs(:, i_MC), delta, dims, mu);
    states(:, :, i_MC) = simulation_smoother(Y_filtering, SSM);
    Y_sim(:, :, i_MC)  = simulate_SSM(SSM, T);
end

% Compute level and growth for last signal
signal     = reshape(states(N_c, :, :), [T, n_MC]);
GDP_level  = exp(median(signal, 2)/100);
GDP_growth = 4*median(signal(2:T, :) - signal(1:(T-1), :), 2);

% Compute autocovariance in simulated data for each model
ACovF_sim  = NaN(n_S, n_lag+1, n_MC);
ACorrF_sim = NaN(n_S, n_lag+1, n_MC);
for i_MC = 1:n_MC
    S_tmp = NaN(n_S, T);
    for i_S = 1:n_S, S_tmp(i_S, :) = Y_sim(S_plus(i_S), :, i_MC) - Y_sim(S_minus(i_S), :, i_MC); end
    S_tmp(isnan(S_data)) = NaN;
    for i_lag = 0:n_lag
        for i_S = 1:n_S
            ACovF_sim(i_S, i_lag+1, i_MC)  = auto_cov(S_tmp(i_S, :), i_lag);
            ACorrF_sim(i_S, i_lag+1, i_MC) = ACovF_sim(i_S, i_lag+1, i_MC)/auto_cov(S_tmp(i_S, :), 0);
        end
    end
end
ACovF_sim  = quantile(ACovF_sim, [0.05, 0.50, 0.95], 3);
ACorrF_sim = quantile(ACorrF_sim, [0.05, 0.50, 0.95], 3);

% Save results
results_v0              = struct();
results_v0.settings     = settings;
results_v0.time         = time;
results_v0.date_vintage = date_vintage;
results_v0.Y            = Y;
results_v0.dims         = dims;
results_v0.delta        = delta;
results_v0.prior        = prior;
results_v0.param_Gibbs  = param_Gibbs;
results_v0.ML           = ML;
results_v0.GDP_level    = GDP_level;
results_v0.GDP_growth   = GDP_growth;
results_v0.ACovF_sim    = ACovF_sim;
results_v0.ACorrF_sim   = ACorrF_sim;
save([mat_dir sprintf('estimates_%s.mat', fname)], '-struct', 'results_v0')
else
    results_v0  = load([mat_dir sprintf('estimates_%s.mat', fname)]);
    param_Gibbs = results_v0.param_Gibbs;
end

% Simulate from the prior
rng(1914)
n_param           = size(param_Gibbs, 1);
param_prior       = NaN(n_param, settings.n_MC);
for i_MC = 1:settings.n_MC
    param_prior(:, i_MC) = simulate_prior(prior);
end

% Recover description
description = name_parameter(dims);

% Compute Monte Carlo standard errors for the mean
param_MCSE = NaN(n_param, 2);
for i_param = 1:n_param
    param_MCSE(i_param, 1) = sqrt(compute_LRV(param_Gibbs(i_param, :))/n_MC);    
    param_MCSE(i_param, 2) = corr(param_Gibbs(i_param, 2:n_MC)', param_Gibbs(i_param, 1:(n_MC-1))');    
end

% Display results in a table
table_param       = array2table([round(quantile(param_prior, [0.05, 0.50, 0.95], 2), 3), ...
                                 round(quantile(param_Gibbs, [0.05, 0.50, 0.95], 2), 3), ...
                                 round(param_MCSE(:, 1), 4), round(param_MCSE(:, 2), 2)], ...
    'variablenames', {'LowPrior', 'MedPrior', 'HighPrior', 'LowPost', 'MedPost', 'HighPost', 'MCSE', 'Autocorr'});
table_param.Names = description.param_names_tab;
table_param       = [table_param(:, 'Names'), table_param(:, 1:(end-1))];
display(table_param)
writetable(table_param, [tab_dir sprintf('param_%s.csv', fname)], 'writerownames', true, 'writevariablenames', true)

% Display densities in a plot
n_abs = 1e2;
for i_param = 1:length(description.param_groupnames)
    
    n_G  = description.n_param(i_param);
    fig0 = figure();
    set(fig0, figsize{:})
    
    if (n_G <= 121)
        
        n_G_tmp        = sum(description.n_param(1:(i_param-1)), 2);
        [n_row, n_col] = compute_dimensions(n_G);
        
        for i_G = 1:n_G
            
            % Compute plot data
            abscisa       = linspace(quantile([param_prior(n_G_tmp+i_G, :), param_Gibbs(n_G_tmp+i_G, :)], 0.005, 2), ...
                quantile([param_prior(n_G_tmp+i_G, :), param_Gibbs(n_G_tmp+i_G, :)], 0.995, 2), n_abs)';
            density_prior = ksdensity(param_prior(n_G_tmp+i_G, :)', abscisa);
            density_Gibbs = ksdensity(param_Gibbs(n_G_tmp+i_G, :)', abscisa);
            
            % Execute and format plot
            ax0   = subplot(n_row, n_col, i_G);
            hold(ax0, 'all')
            plot0 = plot(ax0, abscisa, [density_prior, density_Gibbs]);
            area(ax0, abscisa, density_prior, 'facecolor', green, 'facealpha', 0.2)
            area(ax0, abscisa, density_Gibbs, 'facecolor', dark_green, 'facealpha', 0.5)
            hold(ax0, 'off')
            set(ax0, ax_fonts{:})
            set(ax0, 'box', 'on')
            axis(ax0, 'tight')
            ylabel(ax0, description.param_names_fig{n_G_tmp+i_G, 1}, 'Interpreter', 'TeX')
            set(plot0, {'color'}, {green; dark_green})

        end
                
    end
    
    print(fig0, [fig_dir sprintf('param_%s_%s.eps', fname, description.param_groupnames{i_param})], '-deps')
    
end


%% GIBBS SAMPLING (V1: DIAGONAL VAR + FACTOR COVARIANCES)

close all

% Change path
rmpath(['..' filesep 'simulation' filesep fname filesep 'functions'])
fname = 'v1';
addpath(['..' filesep 'simulation' filesep fname filesep 'functions'])

% Set prior
prior    = construct_prior(dims);
mu       = mean(Y(:, 1), 'omitnan')*ones(N_c, 1);
prior.mu = mu;

if (re_estimate == true) || (~exist([mat_dir sprintf('estimates_%s.mat', fname)], 'file'))
% Set random number generator seed
rng(1914)

% Run Gibbs sampler
[param_Gibbs, states, ML] = Gibbs_sampler(Y_vintage, delta, prior, settings);

% Carry signal extraction and data simulation
Y_sim = NaN(N_E+N_I, T, n_MC);
for i_MC = 1:n_MC
    if isfield(prior, 'mu'), mu = prior.mu; else, mu = mean(Y(:, 1), 'omitnan')*ones(N_c, 1); end
    SSM                = construct_SSM(param_Gibbs(:, i_MC), delta, dims, mu);
    states(:, :, i_MC) = simulation_smoother(Y_filtering, SSM);
    Y_sim(:, :, i_MC)  = simulate_SSM(SSM, T);
end

% Compute level and growth for last signal
signal     = reshape(states(N_c, :, :), [T, n_MC]);
GDP_level  = exp(median(signal, 2)/100);
GDP_growth = 4*median(signal(2:T, :) - signal(1:(T-1), :), 2);

% Compute autocovariance in simulated data for each model
ACovF_sim  = NaN(n_S, n_lag+1, n_MC);
ACorrF_sim = NaN(n_S, n_lag+1, n_MC);
for i_MC = 1:n_MC
    S_tmp = NaN(n_S, T);
    for i_S = 1:n_S, S_tmp(i_S, :) = Y_sim(S_plus(i_S), :, i_MC) - Y_sim(S_minus(i_S), :, i_MC); end
    S_tmp(isnan(S_data)) = NaN;
    for i_lag = 0:n_lag
        for i_S = 1:n_S
            ACovF_sim(i_S, i_lag+1, i_MC)  = auto_cov(S_tmp(i_S, :), i_lag);
            ACorrF_sim(i_S, i_lag+1, i_MC) = ACovF_sim(i_S, i_lag+1, i_MC)/auto_cov(S_tmp(i_S, :), 0);
        end
    end
end
ACovF_sim  = quantile(ACovF_sim, [0.05, 0.50, 0.95], 3);
ACorrF_sim = quantile(ACorrF_sim, [0.05, 0.50, 0.95], 3);

% Save results
results_v1              = struct();
results_v1.settings     = settings;
results_v1.time         = time;
results_v1.date_vintage = date_vintage;
results_v1.Y            = Y;
results_v1.dims         = dims;
results_v1.delta        = delta;
results_v1.prior        = prior;
results_v1.param_Gibbs  = param_Gibbs;
results_v1.ML           = ML;
results_v1.GDP_level    = GDP_level;
results_v1.GDP_growth   = GDP_growth;
results_v1.ACovF_sim    = ACovF_sim;
results_v1.ACorrF_sim   = ACorrF_sim;
save([mat_dir sprintf('estimates_%s.mat', fname)], '-struct', 'results_v1')
else
    results_v1  = load([mat_dir sprintf('estimates_%s.mat', fname)]);
    param_Gibbs = results_v1.param_Gibbs;
end

% Simulate from the prior
rng(1914)
n_param           = size(param_Gibbs, 1);
param_prior       = NaN(n_param, settings.n_MC);
for i_MC = 1:settings.n_MC
    param_prior(:, i_MC) = simulate_prior(prior);
end

% Recover description
description = name_parameter(dims);

% Compute Monte Carlo standard errors for the mean
param_MCSE = NaN(n_param, 2);
for i_param = 1:n_param
    param_MCSE(i_param, 1) = sqrt(compute_LRV(param_Gibbs(i_param, :))/n_MC);    
    param_MCSE(i_param, 2) = corr(param_Gibbs(i_param, 2:n_MC)', param_Gibbs(i_param, 1:(n_MC-1))');    
end

% Display results in a table
table_param       = array2table([round(quantile(param_prior, [0.05, 0.50, 0.95], 2), 3), ...
                                 round(quantile(param_Gibbs, [0.05, 0.50, 0.95], 2), 3), ...
                                 round(param_MCSE(:, 1), 4), round(param_MCSE(:, 2), 2)], ...
    'variablenames', {'LowPrior', 'MedPrior', 'HighPrior', 'LowPost', 'MedPost', 'HighPost', 'MCSE', 'Autocorr'});
table_param.Names = description.param_names_tab;
table_param       = [table_param(:, 'Names'), table_param(:, 1:(end-1))];
display(table_param)
writetable(table_param, [tab_dir sprintf('param_%s.csv', fname)], 'writerownames', true, 'writevariablenames', true)

% Display densities in a plot
n_abs = 1e2;
for i_param = 1:length(description.param_groupnames)
    
    n_G  = description.n_param(i_param);
    fig0 = figure();
    set(fig0, figsize{:})
    
    if (n_G <= 121)
        
        n_G_tmp        = sum(description.n_param(1:(i_param-1)), 2);
        [n_row, n_col] = compute_dimensions(n_G);
        
        for i_G = 1:n_G
            
            % Compute plot data
            abscisa       = linspace(quantile([param_prior(n_G_tmp+i_G, :), param_Gibbs(n_G_tmp+i_G, :)], 0.005, 2), ...
                quantile([param_prior(n_G_tmp+i_G, :), param_Gibbs(n_G_tmp+i_G, :)], 0.995, 2), n_abs)';
            density_prior = ksdensity(param_prior(n_G_tmp+i_G, :)', abscisa);
            density_Gibbs = ksdensity(param_Gibbs(n_G_tmp+i_G, :)', abscisa);
            
            % Execute and format plot
            ax0   = subplot(n_row, n_col, i_G);
            hold(ax0, 'all')
            plot0 = plot(ax0, abscisa, [density_prior, density_Gibbs]);
            area(ax0, abscisa, density_prior, 'facecolor', green, 'facealpha', 0.2)
            area(ax0, abscisa, density_Gibbs, 'facecolor', dark_green, 'facealpha', 0.5)
            hold(ax0, 'off')
            set(ax0, ax_fonts{:})
            set(ax0, 'box', 'on')
            axis(ax0, 'tight')
            ylabel(ax0, description.param_names_fig{n_G_tmp+i_G, 1}, 'Interpreter', 'TeX')
            set(plot0, {'color'}, {green; dark_green})

        end
                
    end
    
    print(fig0, [fig_dir sprintf('param_%s_%s.eps', fname, description.param_groupnames{i_param})], '-deps')
    
end


%% GIBBS SAMPLING (V2: UNRESTRICTED VAR + UNRESTRICTED COVARIANCES)

close all

% Change path
rmpath(['..' filesep 'simulation' filesep fname filesep 'functions'])
fname = 'v2';
addpath(['..' filesep 'simulation' filesep fname filesep 'functions'])

% Set prior
prior    = construct_prior(dims);
mu       = mean(Y(:, 1), 'omitnan')*ones(N_c, 1);
prior.mu = mu;

if (re_estimate == true) || (~exist([mat_dir sprintf('estimates_%s.mat', fname)], 'file'))
% Set random number generator seed
rng(1914)

% Run Gibbs sampler
[param_Gibbs, states, ML] = Gibbs_sampler(Y_vintage, delta, prior, settings);

% Carry signal extraction and data simulation
Y_sim = NaN(N_E+N_I, T, n_MC);
for i_MC = 1:n_MC
    if isfield(prior, 'mu'), mu = prior.mu; else, mu = mean(Y(:, 1), 'omitnan')*ones(N_c, 1); end
    SSM                = construct_SSM(param_Gibbs(:, i_MC), delta, dims, mu);
    states(:, :, i_MC) = simulation_smoother(Y_filtering, SSM);
    Y_sim(:, :, i_MC)  = simulate_SSM(SSM, T);
end

% Compute level and growth for last signal
signal     = reshape(states(N_c, :, :), [T, n_MC]);
GDP_level  = exp(median(signal, 2)/100);
GDP_growth = 4*median(signal(2:T, :) - signal(1:(T-1), :), 2);

% Compute autocovariance in simulated data for each model
ACovF_sim  = NaN(n_S, n_lag+1, n_MC);
ACorrF_sim = NaN(n_S, n_lag+1, n_MC);
for i_MC = 1:n_MC
    S_tmp = NaN(n_S, T);
    for i_S = 1:n_S, S_tmp(i_S, :) = Y_sim(S_plus(i_S), :, i_MC) - Y_sim(S_minus(i_S), :, i_MC); end
    S_tmp(isnan(S_data)) = NaN;
    for i_lag = 0:n_lag
        for i_S = 1:n_S
            ACovF_sim(i_S, i_lag+1, i_MC)  = auto_cov(S_tmp(i_S, :), i_lag);
            ACorrF_sim(i_S, i_lag+1, i_MC) = ACovF_sim(i_S, i_lag+1, i_MC)/auto_cov(S_tmp(i_S, :), 0);
        end
    end
end
ACovF_sim  = quantile(ACovF_sim, [0.05, 0.50, 0.95], 3);
ACorrF_sim = quantile(ACorrF_sim, [0.05, 0.50, 0.95], 3);

% Save results
results_v2              = struct();
results_v2.settings     = settings;
results_v2.time         = time;
results_v2.date_vintage = date_vintage;
results_v2.Y            = Y;
results_v2.dims         = dims;
results_v2.delta        = delta;
results_v2.prior        = prior;
results_v2.param_Gibbs  = param_Gibbs;
results_v2.ML           = ML;
results_v2.GDP_level    = GDP_level;
results_v2.GDP_growth   = GDP_growth;
results_v2.ACovF_sim    = ACovF_sim;
results_v2.ACorrF_sim   = ACorrF_sim;
save([mat_dir sprintf('estimates_%s.mat', fname)], '-struct', 'results_v2')
else
    results_v2  = load([mat_dir sprintf('estimates_%s.mat', fname)]);
    param_Gibbs = results_v2.param_Gibbs;
end

% Simulate from the prior
rng(1914)
n_param           = size(param_Gibbs, 1);
param_prior       = NaN(n_param, settings.n_MC);
for i_MC = 1:settings.n_MC
    param_prior(:, i_MC) = simulate_prior(prior);
end

% Recover description
description = name_parameter(dims);

% Compute Monte Carlo standard errors for the mean
param_MCSE = NaN(n_param, 2);
for i_param = 1:n_param
    param_MCSE(i_param, 1) = sqrt(compute_LRV(param_Gibbs(i_param, :))/n_MC);    
    param_MCSE(i_param, 2) = corr(param_Gibbs(i_param, 2:n_MC)', param_Gibbs(i_param, 1:(n_MC-1))');    
end

% Display results in a table
table_param       = array2table([round(quantile(param_prior, [0.05, 0.50, 0.95], 2), 3), ...
                                 round(quantile(param_Gibbs, [0.05, 0.50, 0.95], 2), 3), ...
                                 round(param_MCSE(:, 1), 4), round(param_MCSE(:, 2), 2)], ...
    'variablenames', {'LowPrior', 'MedPrior', 'HighPrior', 'LowPost', 'MedPost', 'HighPost', 'MCSE', 'Autocorr'});
table_param.Names = description.param_names_tab;
table_param       = [table_param(:, 'Names'), table_param(:, 1:(end-1))];
display(table_param)
writetable(table_param, [tab_dir sprintf('param_%s.csv', fname)], 'writerownames', true, 'writevariablenames', true)

% Display densities in a plot
n_abs = 1e2;
for i_param = 1:length(description.param_groupnames)
    
    n_G  = description.n_param(i_param);
    fig0 = figure();
    set(fig0, figsize{:})
    
    if (n_G <= 121)
        
        n_G_tmp        = sum(description.n_param(1:(i_param-1)), 2);
        [n_row, n_col] = compute_dimensions(n_G);
        
        for i_G = 1:n_G
            
            % Compute plot data
            abscisa       = linspace(quantile([param_prior(n_G_tmp+i_G, :), param_Gibbs(n_G_tmp+i_G, :)], 0.005, 2), ...
                quantile([param_prior(n_G_tmp+i_G, :), param_Gibbs(n_G_tmp+i_G, :)], 0.995, 2), n_abs)';
            density_prior = ksdensity(param_prior(n_G_tmp+i_G, :)', abscisa);
            density_Gibbs = ksdensity(param_Gibbs(n_G_tmp+i_G, :)', abscisa);
            
            % Execute and format plot
            ax0   = subplot(n_row, n_col, i_G);
            hold(ax0, 'all')
            plot0 = plot(ax0, abscisa, [density_prior, density_Gibbs]);
            area(ax0, abscisa, density_prior, 'facecolor', green, 'facealpha', 0.2)
            area(ax0, abscisa, density_Gibbs, 'facecolor', dark_green, 'facealpha', 0.5)
            hold(ax0, 'off')
            set(ax0, ax_fonts{:})
            set(ax0, 'box', 'on')
            axis(ax0, 'tight')
            ylabel(ax0, description.param_names_fig{n_G_tmp+i_G, 1}, 'Interpreter', 'TeX')
            set(plot0, {'color'}, {green; dark_green})

        end
                
    end
    
    print(fig0, [fig_dir sprintf('param_%s_%s.eps', fname, description.param_groupnames{i_param})], '-deps')
    
end


%% GIBBS SAMPLING (V3: UNRESTRICTED VAR + FACTOR COVARIANCES)

close all

% Change path
rmpath(['..' filesep 'simulation' filesep fname filesep 'functions'])
fname = 'v3';
addpath(['..' filesep 'simulation' filesep fname filesep 'functions'])

% Set prior
prior    = construct_prior(dims);
mu       = mean(Y(:, 1), 'omitnan')*ones(N_c, 1);
prior.mu = mu;

if (re_estimate == true) || (~exist([mat_dir sprintf('estimates_%s.mat', fname)], 'file'))
% Set random number generator seed
rng(1914)

% Run Gibbs sampler
[param_Gibbs, states, ML] = Gibbs_sampler(Y_vintage, delta, prior, settings);

% Carry signal extraction and data simulation
Y_sim = NaN(N_E+N_I, T, n_MC);
for i_MC = 1:n_MC
    if isfield(prior, 'mu'), mu = prior.mu; else, mu = mean(Y(:, 1), 'omitnan')*ones(N_c, 1); end
    SSM                = construct_SSM(param_Gibbs(:, i_MC), delta, dims, mu);
    states(:, :, i_MC) = simulation_smoother(Y_filtering, SSM);
    Y_sim(:, :, i_MC)  = simulate_SSM(SSM, T);
end

% Compute level and growth for last signal
signal     = reshape(states(N_c, :, :), [T, n_MC]);
GDP_level  = exp(median(signal, 2)/100);
GDP_growth = 4*median(signal(2:T, :) - signal(1:(T-1), :), 2);

% Compute autocovariance in simulated data for each model
ACovF_sim  = NaN(n_S, n_lag+1, n_MC);
ACorrF_sim = NaN(n_S, n_lag+1, n_MC);
for i_MC = 1:n_MC
    S_tmp = NaN(n_S, T);
    for i_S = 1:n_S, S_tmp(i_S, :) = Y_sim(S_plus(i_S), :, i_MC) - Y_sim(S_minus(i_S), :, i_MC); end
    S_tmp(isnan(S_data)) = NaN;
    for i_lag = 0:n_lag
        for i_S = 1:n_S
            ACovF_sim(i_S, i_lag+1, i_MC)  = auto_cov(S_tmp(i_S, :), i_lag);
            ACorrF_sim(i_S, i_lag+1, i_MC) = ACovF_sim(i_S, i_lag+1, i_MC)/auto_cov(S_tmp(i_S, :), 0);
        end
    end
end
ACovF_sim  = quantile(ACovF_sim, [0.05, 0.50, 0.95], 3);
ACorrF_sim = quantile(ACorrF_sim, [0.05, 0.50, 0.95], 3);

% Save results
results_v3              = struct();
results_v3.settings     = settings;
results_v3.time         = time;
results_v3.date_vintage = date_vintage;
results_v3.Y            = Y;
results_v3.dims         = dims;
results_v3.delta        = delta;
results_v3.prior        = prior;
results_v3.param_Gibbs  = param_Gibbs;
results_v3.ML           = ML;
results_v3.GDP_level    = GDP_level;
results_v3.GDP_growth   = GDP_growth;
results_v3.ACovF_sim    = ACovF_sim;
results_v3.ACorrF_sim   = ACorrF_sim;
save([mat_dir sprintf('estimates_%s.mat', fname)], '-struct', 'results_v3')
else
    results_v3  = load([mat_dir sprintf('estimates_%s.mat', fname)]);
    param_Gibbs = results_v3.param_Gibbs;
end

% Simulate from the prior
n_param           = size(param_Gibbs, 1);
param_prior       = NaN(n_param, settings.n_MC);
for i_MC = 1:settings.n_MC
    param_prior(:, i_MC) = simulate_prior(prior);
end

% Recover description
description = name_parameter(dims);

% Compute Monte Carlo standard errors for the mean
rng(1914)
param_MCSE = NaN(n_param, 2);
for i_param = 1:n_param
    param_MCSE(i_param, 1) = sqrt(compute_LRV(param_Gibbs(i_param, :))/n_MC);    
    param_MCSE(i_param, 2) = corr(param_Gibbs(i_param, 2:n_MC)', param_Gibbs(i_param, 1:(n_MC-1))');    
end

% Display results in a table
table_param       = array2table([round(quantile(param_prior, [0.05, 0.50, 0.95], 2), 3), ...
                                 round(quantile(param_Gibbs, [0.05, 0.50, 0.95], 2), 3), ...
                                 round(param_MCSE(:, 1), 4), round(param_MCSE(:, 2), 2)], ...
    'variablenames', {'LowPrior', 'MedPrior', 'HighPrior', 'LowPost', 'MedPost', 'HighPost', 'MCSE', 'Autocorr'});
table_param.Names = description.param_names_tab;
table_param       = [table_param(:, 'Names'), table_param(:, 1:(end-1))];
display(table_param)
writetable(table_param, [tab_dir sprintf('param_%s.csv', fname)], 'writerownames', true, 'writevariablenames', true)

% Display densities in a plot
n_abs = 1e2;
for i_param = 1:length(description.param_groupnames)
    
    n_G  = description.n_param(i_param);
    fig0 = figure();
    set(fig0, figsize{:})
    
    if (n_G <= 121)
        
        n_G_tmp        = sum(description.n_param(1:(i_param-1)), 2);
        [n_row, n_col] = compute_dimensions(n_G);
        
        for i_G = 1:n_G
            
            % Compute plot data
            abscisa       = linspace(quantile([param_prior(n_G_tmp+i_G, :), param_Gibbs(n_G_tmp+i_G, :)], 0.005, 2), ...
                quantile([param_prior(n_G_tmp+i_G, :), param_Gibbs(n_G_tmp+i_G, :)], 0.995, 2), n_abs)';
            density_prior = ksdensity(param_prior(n_G_tmp+i_G, :)', abscisa);
            density_Gibbs = ksdensity(param_Gibbs(n_G_tmp+i_G, :)', abscisa);
            
            % Execute and format plot
            ax0   = subplot(n_row, n_col, i_G);
            hold(ax0, 'all')
            plot0 = plot(ax0, abscisa, [density_prior, density_Gibbs]);
            area(ax0, abscisa, density_prior, 'facecolor', green, 'facealpha', 0.2)
            area(ax0, abscisa, density_Gibbs, 'facecolor', dark_green, 'facealpha', 0.5)
            hold(ax0, 'off')
            set(ax0, ax_fonts{:})
            set(ax0, 'box', 'on')
            axis(ax0, 'tight')
            ylabel(ax0, description.param_names_fig{n_G_tmp+i_G, 1}, 'Interpreter', 'TeX')
            set(plot0, {'color'}, {green; dark_green})

        end
                
    end
    
    print(fig0, [fig_dir sprintf('param_%s_%s.eps', fname, description.param_groupnames{i_param})], '-deps')
    
end


%% GIBBS SAMPLING (V4: TREND+CYCLE)

close all

% Change path
rmpath(['..' filesep 'simulation' filesep fname filesep 'functions'])
fname = 'v4';
addpath(['..' filesep 'simulation' filesep fname filesep 'functions'])

% Set prior
prior    = construct_prior(dims);
mu       = mean(Y(:, 1), 'omitnan')*ones(N_c, 1);
prior.mu = mu;

if (re_estimate == true) || (~exist([mat_dir sprintf('estimates_%s.mat', fname)], 'file'))
% Set random number generator seed
rng(1914)

% Run Gibbs sampler
[param_Gibbs, states, ML] = Gibbs_sampler(Y_vintage, delta, prior, settings);

% Carry signal extraction and data simulation
Y_sim = NaN(N_E+N_I, T, n_MC);
for i_MC = 1:n_MC
    if isfield(prior, 'mu'), mu = prior.mu; else, mu = mean(Y(:, 1), 'omitnan')*ones(N_c, 1); end
    SSM                = construct_SSM(param_Gibbs(:, i_MC), delta, dims, mu);
    states(:, :, i_MC) = simulation_smoother(Y_filtering, SSM);
    Y_sim(:, :, i_MC)  = simulate_SSM(SSM, T);
end

% Compute level and growth for last signal
trend      = reshape(states(N_c, :, :), [T, n_MC]);
cycle      = reshape(states(2*N_c+1, :, :), [T, n_MC]);
signal     = trend+cycle;
GDP_trend  = exp(median(trend, 2)/100);
GDP_cycle  = median(cycle, 2);
GDP_level  = exp(median(signal, 2)/100);
GDP_growth = 4*median(signal(2:T, :) - signal(1:(T-1), :), 2);

% Compute autocovariance in simulated data for each model
ACovF_sim  = NaN(n_S, n_lag+1, n_MC);
ACorrF_sim = NaN(n_S, n_lag+1, n_MC);
for i_MC = 1:n_MC
    S_tmp = NaN(n_S, T);
    for i_S = 1:n_S, S_tmp(i_S, :) = Y_sim(S_plus(i_S), :, i_MC) - Y_sim(S_minus(i_S), :, i_MC); end
    S_tmp(isnan(S_data)) = NaN;
    for i_lag = 0:n_lag
        for i_S = 1:n_S
            ACovF_sim(i_S, i_lag+1, i_MC)  = auto_cov(S_tmp(i_S, :), i_lag);
            ACorrF_sim(i_S, i_lag+1, i_MC) = ACovF_sim(i_S, i_lag+1, i_MC)/auto_cov(S_tmp(i_S, :), 0);
        end
    end
end
ACovF_sim  = quantile(ACovF_sim, [0.05, 0.50, 0.95], 3);
ACorrF_sim = quantile(ACorrF_sim, [0.05, 0.50, 0.95], 3);

% Save results
results_v4              = struct();
results_v4.settings     = settings;
results_v4.time         = time;
results_v4.date_vintage = date_vintage;
results_v4.Y            = Y;
results_v4.dims         = dims;
results_v4.delta        = delta;
results_v4.prior        = prior;
results_v4.param_Gibbs  = param_Gibbs;
results_v4.ML           = ML;
results_v4.GDP_trend    = GDP_trend;
results_v4.GDP_cycle    = GDP_cycle;
results_v4.GDP_level    = GDP_level;
results_v4.GDP_growth   = GDP_growth;
results_v4.ACovF_sim    = ACovF_sim;
results_v4.ACorrF_sim   = ACorrF_sim;
save([mat_dir sprintf('estimates_%s.mat', fname)], '-struct', 'results_v4')
else
    results_v4  = load([mat_dir sprintf('estimates_%s.mat', fname)]);
    param_Gibbs = results_v4.param_Gibbs;
end

% Simulate from the prior
rng(1914)
n_param           = size(param_Gibbs, 1);
param_prior       = NaN(n_param, settings.n_MC);
for i_MC = 1:settings.n_MC
    param_prior(:, i_MC) = simulate_prior(prior);
end

% Recover description
description = name_parameter(dims);

% Compute Monte Carlo standard errors for the mean
param_MCSE = NaN(n_param, 2);
for i_param = 1:n_param
    param_MCSE(i_param, 1) = sqrt(compute_LRV(param_Gibbs(i_param, :))/n_MC);    
    param_MCSE(i_param, 2) = corr(param_Gibbs(i_param, 2:n_MC)', param_Gibbs(i_param, 1:(n_MC-1))');    
end

% Display results in a table
table_param       = array2table([round(quantile(param_prior, [0.05, 0.50, 0.95], 2), 3), ...
                                 round(quantile(param_Gibbs, [0.05, 0.50, 0.95], 2), 3), ...
                                 round(param_MCSE(:, 1), 4), round(param_MCSE(:, 2), 2)], ...
    'variablenames', {'LowPrior', 'MedPrior', 'HighPrior', 'LowPost', 'MedPost', 'HighPost', 'MCSE', 'Autocorr'});
table_param.Names = description.param_names_tab;
table_param       = [table_param(:, 'Names'), table_param(:, 1:(end-1))];
display(table_param)
writetable(table_param, [tab_dir sprintf('param_%s.csv', fname)], 'writerownames', true, 'writevariablenames', true)

% Display densities in a plot
n_abs = 1e2;
for i_param = 1:length(description.param_groupnames)
    
    n_G  = description.n_param(i_param);
    fig0 = figure();
    set(fig0, figsize{:})
    
    if (n_G <= 121)
        
        n_G_tmp        = sum(description.n_param(1:(i_param-1)), 2);
        [n_row, n_col] = compute_dimensions(n_G);
        
        for i_G = 1:n_G
            
            % Compute plot data
            abscisa       = linspace(quantile([param_prior(n_G_tmp+i_G, :), param_Gibbs(n_G_tmp+i_G, :)], 0.005, 2), ...
                quantile([param_prior(n_G_tmp+i_G, :), param_Gibbs(n_G_tmp+i_G, :)], 0.995, 2), n_abs)';
            density_prior = ksdensity(param_prior(n_G_tmp+i_G, :)', abscisa);
            density_Gibbs = ksdensity(param_Gibbs(n_G_tmp+i_G, :)', abscisa);
            
            % Execute and format plot
            ax0   = subplot(n_row, n_col, i_G);
            hold(ax0, 'all')
            plot0 = plot(ax0, abscisa, [density_prior, density_Gibbs]);
            area(ax0, abscisa, density_prior, 'facecolor', green, 'facealpha', 0.2)
            area(ax0, abscisa, density_Gibbs, 'facecolor', dark_green, 'facealpha', 0.5)
            hold(ax0, 'off')
            set(ax0, ax_fonts{:})
            set(ax0, 'box', 'on')
            axis(ax0, 'tight')
            ylabel(ax0, description.param_names_fig{n_G_tmp+i_G, 1}, 'Interpreter', 'TeX')
            set(plot0, {'color'}, {green; dark_green})

        end
                
    end
    
    print(fig0, [fig_dir sprintf('param_%s_%s.eps', fname, description.param_groupnames{i_param})], '-deps')
    
end


%% COMPARISON OF IMPLIED GDP SOLERA SERIES

close all

% Change path
rmpath(['..' filesep 'simulation' filesep fname filesep 'functions'])
fname = 'v0';
addpath(['..' filesep 'simulation' filesep fname filesep 'functions'])

% Upload results
results_v0 = load([mat_dir 'estimates_v0.mat']); 
results_v1 = load([mat_dir 'estimates_v1.mat']); 
results_v2 = load([mat_dir 'estimates_v2.mat']); 
results_v3 = load([mat_dir 'estimates_v3.mat']); 
results_v4 = load([mat_dir 'estimates_v4.mat']); 

% Set model names
model_name  = {'Diagonal VAR + Unrestricted covariances', 'Diagonal VAR + Factor covariances', 'Unrestricted VAR + Unrestricted covariances', 'Unrestricted + Factor covariances'};
model_color = {blue; red; black; dark_green};
model_style = {'-'; '--'; '-.'; ':'};

% Recover GDP solera estimates according to different models
GDP_level_v0  = results_v0.GDP_level;
GDP_growth_v0 = results_v0.GDP_growth;
GDP_level_v1  = results_v1.GDP_level;
GDP_growth_v1 = results_v1.GDP_growth;
GDP_level_v2  = results_v2.GDP_level;
GDP_growth_v2 = results_v2.GDP_growth;
GDP_level_v3  = results_v3.GDP_level;
GDP_growth_v3 = results_v3.GDP_growth;
GDP_trend_v4  = results_v4.GDP_trend;
GDP_cycle_v4  = results_v4.GDP_cycle;
GDP_level_v4  = results_v4.GDP_level;
GDP_growth_v4 = results_v4.GDP_growth;


% Create figure comparing GDP solera across models
fig0  = figure();
ax0   = axes();
plot0 = plot(ax0, time, [GDP_level_v0, GDP_level_v1, GDP_level_v2, GDP_level_v3]);
set(plot0, 'linewidth', 2)
set(plot0, {'linestyle'}, model_style)
set(plot0, {'color'}, model_color)

% Add extras
xlim(ax0, [time(1), time(end)])
ylabel(ax0, 'GDP solera estimates (levels)')
legend(ax0, model_name, 'location', 'southeast', 'orientation', 'vertical') 

% Tune ax handle
set(ax0, ax_fonts{:})
set(ax0, 'box', 'on')
set(ax0, 'linewidth', 0.5)

% Save and close figure
print(fig0, [fig_dir 'comparison_GDP_level.eps'], '-depsc')


% Create figure comparing GDP solera across models
fig0  = figure();
ax0   = axes();
plot0 = plot(ax0, time(2:T), [GDP_growth_v0, GDP_growth_v1, GDP_growth_v2, GDP_growth_v3]);
set(plot0, 'linewidth', 2)
set(plot0, {'linestyle'}, model_style)
set(plot0, {'color'}, model_color)

% Add extras
xlim(ax0, [time(2), time(end)])
ylim(ax0, [-8, 10]);
ylabel(ax0, 'GDP solera estimates (growth)')
line(ax0, [time(2), time(end)], [0, 0], 'linewidth', 0.5, 'linestyle', '--', 'color', [0, 0, 0])
legend(ax0, model_name, 'location', 'southwest', 'orientation', 'vertical') 

% Tune ax handle
set(ax0, ax_fonts{:})
set(ax0, 'box', 'on')
set(ax0, 'linewidth', 0.5)

% Save and close figure
print(fig0, [fig_dir 'comparison_GDP_growth.eps'], '-depsc')


% Create figure comparing GDP solera across models
fig0  = figure();
ax0   = axes();
plot0 = plot(ax0, time, [GDP_level_v2, GDP_level_v4]);
set(plot0, 'linewidth', 2)
set(plot0, {'linestyle'}, model_style(1:2))
set(plot0, {'color'}, model_color(1:2))

% Add extras
xlim(ax0, [time(1), time(end)])
ylabel(ax0, 'GDP solera estimates (levels)')
legend(ax0, {'Baseline', 'Trend+Cycle'}, 'location', 'southeast', 'orientation', 'vertical') 

% Tune ax handle
set(ax0, ax_fonts{:})
set(ax0, 'box', 'on')
set(ax0, 'linewidth', 0.5)

% Save and close figure
print(fig0, [fig_dir 'comparison_GDP_cycle_level.eps'], '-depsc')


% Create figure comparing GDP solera across models
fig0  = figure();
ax0   = axes();
plot0 = plot(ax0, time(2:T), [GDP_growth_v2, GDP_growth_v4]);
set(plot0, 'linewidth', 2)
set(plot0, {'linestyle'}, model_style(1:2))
set(plot0, {'color'}, model_color(1:2))

% Add extras
xlim(ax0, [time(2), time(end)])
ylim(ax0, [-8, 10]);
ylabel(ax0, 'GDP solera estimates (growth)')
line(ax0, [time(2), time(end)], [0, 0], 'linewidth', 0.5, 'linestyle', '--', 'color', [0, 0, 0])
legend(ax0, {'Baseline', 'Trend+Cycle'}, 'location', 'southwest', 'orientation', 'vertical') 

% Tune ax handle
set(ax0, ax_fonts{:})
set(ax0, 'box', 'on')
set(ax0, 'linewidth', 0.5)

% Save and close figure
print(fig0, [fig_dir 'comparison_GDP_cycle_growth.eps'], '-depsc')

% Create figure decomposing GDP solera in trend+cycle
fig0  = figure();
ax0   = axes();
plot0 = plot(ax0, time, [GDP_level_v4, GDP_trend_v4]);
set(plot0, 'linewidth', 2)
set(plot0, 'linestyle', '-')
set(plot0, {'color'}, model_color(3:4))

% Add extras
xlim(ax0, [time(1), time(end)])
ylabel(ax0, 'Trend in GDP estimates')
legend(ax0, {'GDP', 'Trend component'}, 'location', 'southeast', 'orientation', 'vertical') 

% Tune ax handle
set(ax0, ax_fonts{:})
set(ax0, 'box', 'on')
set(ax0, 'linewidth', 0.5)

% Save and close figure
print(fig0, [fig_dir 'decomposition_GDP_trend.eps'], '-depsc')

% Create figure decomposing GDP solera in trend+cycle
fig0  = figure();
ax0   = axes();
plot0 = plot(ax0, time, GDP_cycle_v4);
set(plot0, 'linewidth', 2)
set(plot0, 'linestyle', '-')
set(plot0, 'color', model_color{4})

% Add extras
xlim(ax0, [time(1), time(end)])
ylabel(ax0, 'Cycle in GDP estimates')
line(ax0, [time(1), time(end)], [0, 0], 'linewidth', 0.5, 'linestyle', '--', 'color', [0, 0, 0])

% Tune ax handle
set(ax0, ax_fonts{:})
set(ax0, 'box', 'on')
set(ax0, 'linewidth', 0.5)

% Save and close figure
print(fig0, [fig_dir 'decomposition_GDP_cycle.eps'], '-depsc')


%% MARGINAL LIKELIHOOD COMPARISON

% Recover number of parameters
n_param_0 = size(results_v0.param_Gibbs, 1);
n_param_1 = size(results_v1.param_Gibbs, 1);
n_param_2 = size(results_v2.param_Gibbs, 1);
n_param_3 = size(results_v3.param_Gibbs, 1);
n_param_4 = size(results_v4.param_Gibbs, 1);

% Recover marginal log-likelihoods
ML_0 = results_v0.ML.L_prior + results_v0.ML.L_likelihood - results_v0.ML.L_posterior;
ML_1 = results_v1.ML.L_prior + results_v1.ML.L_likelihood - results_v1.ML.L_posterior;
ML_2 = results_v2.ML.L_prior + results_v2.ML.L_likelihood - results_v2.ML.L_posterior;
ML_3 = results_v3.ML.L_prior + results_v3.ML.L_likelihood - results_v3.ML.L_posterior;
ML_4 = results_v4.ML.L_prior + results_v4.ML.L_likelihood - results_v4.ML.L_posterior;

% Compute point estimates of marginal log-likelihoods
ML_0_mean = mean(ML_0);
ML_1_mean = mean(ML_1);
ML_2_mean = mean(ML_2);
ML_3_mean = mean(ML_3);
ML_4_mean = mean(ML_4);

% Compute numerical standard errors
ML_0_se = sqrt(compute_LRV(ML_0')/length(ML_0));
ML_1_se = sqrt(compute_LRV(ML_1')/length(ML_1));
ML_2_se = sqrt(compute_LRV(ML_2')/length(ML_2));
ML_3_se = sqrt(compute_LRV(ML_3')/length(ML_3));
ML_4_se = sqrt(compute_LRV(ML_4')/length(ML_4));

% Display results in a table
table_fit       = array2table([n_param_0, results_v0.ML.L_likelihood, ML_0_mean, ML_0_se; ...
                               n_param_1, results_v1.ML.L_likelihood, ML_1_mean, ML_1_se; ...
                               n_param_2, results_v2.ML.L_likelihood, ML_2_mean, ML_2_se; ...
                               n_param_3, results_v3.ML.L_likelihood, ML_3_mean, ML_3_se; ...
                               n_param_4, results_v4.ML.L_likelihood, ML_4_mean, ML_4_se], ...
    'variablenames', {'NParam', 'Likelihood', 'MarginalLikelihood', 'MCSE'}, ...
    'rownames', {'Restricted covariance/VAR matrices', 'Restricted covariance matrices', 'Restricted VAR matrix', 'Unrestricted', 'Trend+Cycle'});
display(table_fit)
writetable(table_fit, [tab_dir 'marginal_likelihoods.csv'], 'writerownames', true, 'writevariablenames', true)


%% FIT OF STATISTICAL DISCREPANCY SERIAL DEPENDENCE

% Compute autocovariance in simulated data for each model
ACovF_sim = NaN(n_S, n_lag+1, 3, 5);
ACovF_sim(:, :, :, 1) = results_v0.ACovF_sim;
ACovF_sim(:, :, :, 2) = results_v1.ACovF_sim;
ACovF_sim(:, :, :, 3) = results_v2.ACovF_sim;
ACovF_sim(:, :, :, 4) = results_v3.ACovF_sim;
ACovF_sim(:, :, :, 5) = results_v4.ACovF_sim;

% Compute autocorrelation in simulated data for each model
ACorrF_sim = NaN(n_S, n_lag+1, 3, 5);
ACorrF_sim(:, :, :, 1) = results_v0.ACorrF_sim;
ACorrF_sim(:, :, :, 2) = results_v1.ACorrF_sim;
ACorrF_sim(:, :, :, 3) = results_v2.ACorrF_sim;
ACorrF_sim(:, :, :, 4) = results_v3.ACorrF_sim;
ACorrF_sim(:, :, :, 5) = results_v4.ACorrF_sim;

% Plot autocovariance structure according to different models
for i_v = 0:4
   for i_S = 1:n_S
       plot_data = ACovF_data(i_S, :)';
       plot_sim  = reshape(ACovF_sim(i_S, :, :, i_v+1), [n_lag+1, 3]);
       
       % Create figure comparing GDP solera with last-signal estimate
       fig0  = figure();
       ax0   = axes();
       fill(ax0, [0:n_lag, fliplr(0:n_lag)], [plot_sim(:, 1)', fliplr(plot_sim(:, 3)')], ...
           green, 'facealpha', 0.3, 'linestyle', 'none');
       hold('all')
       plot0 = plot(ax0, 0:n_lag, [plot_sim(:, 2), plot_data]);
       set(plot0, 'linewidth', 2)
       set(plot0, {'linestyle'}, {':'; '-'})
       set(plot0, {'color'}, {dark_green; red})
       hold('off')
       
       % Add extras
       xlim(ax0, [0, n_lag])
       xticks(ax0, 0:n_lag)
       xlabel(ax0, 'Lag')
       ylabel(ax0, ['ACF of ' S_name{i_S}])
       line(ax0, [0, n_lag], [0, 0], 'linewidth', 0.5, 'linestyle', '--', 'color', [0, 0, 0])
       legend(plot0, 'Model', 'Data', 'location', 'best', 'orientation', 'vertical')
       
       % Tune ax handle
       set(ax0, ax_fonts{:})
       set(ax0, 'box', 'on')
       set(ax0, 'linewidth', 0.5)
       
       % Save and close figure
       print(fig0, [fig_dir sprintf('discrepancies_autocov_v%d_%s.eps', i_v, regexprep(S_name{i_S}, ' ', '-'))], '-depsc')
   end
end

% Plot autocovariance structure according to different models
for i_v = 0:4
   for i_S = 1:n_S
       plot_data = ACorrF_data(i_S, :)';
       plot_sim  = reshape(ACorrF_sim(i_S, :, :, i_v+1), [n_lag+1, 3]);
       
       % Create figure comparing GDP solera with last-signal estimate
       fig0  = figure();
       ax0   = axes();
       fill(ax0, [0:n_lag, fliplr(0:n_lag)], [plot_sim(:, 1)', fliplr(plot_sim(:, 3)')], ...
           green, 'facealpha', 0.3, 'linestyle', 'none');
       hold('all')
       plot0 = plot(ax0, 0:n_lag, [plot_sim(:, 2), plot_data]);
       set(plot0, 'linewidth', 2)
       set(plot0, {'linestyle'}, {':'; '-'})
       set(plot0, {'color'}, {dark_green; red})
       hold('off')
       
       % Add extras
       xlim(ax0, [0, n_lag])
       xticks(ax0, 0:n_lag)
       xlabel(ax0, 'Lag')
       ylabel(ax0, ['ACF of ' S_name{i_S}])
       line(ax0, [0, n_lag], [0, 0], 'linewidth', 0.5, 'linestyle', '--', 'color', [0, 0, 0])
       legend(plot0, 'Model', 'Data', 'location', 'best', 'orientation', 'vertical')
       
       % Tune ax handle
       set(ax0, ax_fonts{:})
       set(ax0, 'box', 'on')
       set(ax0, 'linewidth', 0.5)
       
       % Save and close figure
       print(fig0, [fig_dir sprintf('discrepancies_autocorr_v%d_%s.eps', i_v, regexprep(S_name{i_S}, ' ', '-'))], '-depsc')
   end
end


%% LOCAL FUNCTIONS

% Compute LRV using parametric VAR model
function LRV = compute_LRV(Y, nLag)

[N, T] = size(Y);
if (nargin < 2), nLag = ceil(0.75*(T^(1/3))); end
y      = Y(:, (nLag+1):T);
x      = [ones(1, T-nLag); NaN(N*nLag, T-nLag)];
for iLag = 1:nLag
    x((2+N*(iLag-1)):(1+N*iLag), :) = Y(:, (nLag+1-iLag):(T-iLag));
end
B      = y/x;
U      = y - B*x;
S      = U*(U')/((T-nLag)-(1+N*nLag));
IB     = pinv(eye(N) - sum(reshape(B(:, 2:(1+N*nLag)), [N, N, nLag]), 3));
LRV    = IB*S*(IB');

end

% Compute dimensions of subplot axes
function [nRow, nColumn] = compute_dimensions(nParam)

nRowMax    = 10;
nColumnMax = 13;
matrixTemp = ((1:nRowMax)') + (1:nColumnMax);
matrixTemp(((1:nRowMax)') .* (1:nColumnMax) < nParam) = Inf;
matrixTemp(((1:nRowMax)') - ((1+sqrt(5))/2)*(1:nColumnMax) > 0) = Inf;
[matrixTemp, nRow] = min(matrixTemp, [], 1);
[~, nColumn]       = min(matrixTemp);
nRow               = nRow(1, nColumn);

end
