%%% FIG-REVISIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Clear memory
clear
clc
close all

% Set filename
fname = '';

% Add paths and directories
addpath(['..' filesep 'simulation' filesep 'v0' filesep 'functions'])
data_dir = ['..' filesep 'data' filesep 'output' filesep];
mat_dir  = ['output' filesep 'matfiles' filesep];
fig_dir  = ['output' filesep 'figures' filesep];

% Recover data and estimates
datasets  = load([data_dir 'data.mat']);
estimates = load([mat_dir sprintf('estimates_real_time%s.mat', fname)]);

% Extract objects for plot
real_time    = estimates.real_time;
time         = estimates.time;
state_save   = estimates.state_save;
state_median = squeeze(state_save(:, :, 50, :));

% Determine relevant signal
date_signal  = vintage2date( datasets.GDE.compr_aux );
n_month      = length(real_time);
signal_index = ones(n_month, 1);
for i_c = 2:length(date_signal)
    signal_index(real_time >= date_signal(i_c)) = i_c;
end

% Extract GDE, GDI and GDO data
GDE_data      = readtable([data_dir 'GDE_raw.csv']);
GDI_data      = readtable([data_dir 'GDI_raw.csv']);
GDP_time      = cellfun(@(s) datetime(str2double(s(1:4)), 3*str2double(s(6))-2, 1), GDE_data{:, 1});
GDE_growth    = GDE_data{:, 2:end};
GDE_growth    = [NaN(1, size(GDE_growth, 2)); 100*((GDE_growth(2:end, :)./GDE_growth(1:(end-1), :)).^4-1)];
GDE_growth    = GDE_growth((GDP_time >= min(time)) & (GDP_time <= max(time)), :);
GDI_growth    = GDI_data{:, 2:end};
GDI_growth    = [NaN(1, size(GDI_growth, 2)); 100*((GDI_growth(2:end, :)./GDI_growth(1:(end-1), :)).^4-1)];
GDI_growth    = GDI_growth((GDP_time >= min(time)) & (GDP_time <= max(time)), :);
GDO_growth    = NaN(size(GDE_data{:, 2:end}, 1), size(GDE_data{:, 2:end}, 2), 2);
GDO_growth(:, :, 1) = GDE_data{:, 2:end};
GDO_growth(:, :, 2) = GDI_data{:, 2:end};
GDO_growth    = mean(GDO_growth, 3, 'omitnan');
GDO_growth    = [NaN(1, size(GDO_growth, 2)); 100*((GDO_growth(2:end, :)./GDO_growth(1:(end-1), :)).^4-1)];
GDO_growth    = GDO_growth((GDP_time >= min(time)) & (GDP_time <= max(time)), :);
GDP_time      = GDP_time((GDP_time >= min(time)) & (GDP_time <= max(time)), :);
GDP_real_time = datetime(2002, 9-1+(1:size(GDE_growth, 2))', 1);

% Match GDPPlus and GDO releases
GDO_growth    = GDO_growth(:, ismember(GDP_real_time, real_time));
GDP_real_time = GDP_real_time(ismember(GDP_real_time, real_time));

% Extract GDPplus data
GDPplus_all       = xlsread([data_dir 'GDPplus_raw.xlsx']);
GDPplus_time      = datetime(1960, 3*(1:size(GDPplus_all, 1))-2, 1)';
GDPplus_real_time = datetime(2013, 7+([1:65, 67:size(GDPplus_all, 2)+1]), 1)';
GDPplus_all       = GDPplus_all(:, GDPplus_real_time <= max(GDP_real_time));
GDPplus_real_time = GDPplus_real_time(GDPplus_real_time <= max(GDP_real_time));

% Inputs for plots and tables
ax_fonts   = {'fontname', 'times', 'fontsize', 10, 'linewidth', 0.5};
black      = [  0,   0,   0];
blue       = [ 51,  51, 153]/255;
red        = [153,  51,  51]/255;
orange     = [242, 133,   0]/255;
green      = [ 51, 153,  51]/255;
dark_green = [  0, 100,   0]/255;
grey       = [ 51,  51,  51]/255;


%% CONSTRUCT VINTAGES OF GDPSOLERA, GDPPLUS, GDO

% Restrict sample
signal_index = signal_index((year(real_time) >= 2017) & (year(real_time) <= 2022));
state_median = state_median(:, (year(time) >= 1984) & (year(time) <= 2019), (year(real_time) >= 2017) & (year(real_time) <= 2022));
GDPplus      = GDPplus_all((year(GDPplus_time) >= 1984) & (year(GDPplus_time) <= 2019), (year(GDPplus_real_time) >= 2017) & (year(GDPplus_real_time) <= 2022));
GDO          = GDO_growth((year(time) >= 1984) & (year(time) <= 2019), (year(real_time) >= 2017) & (year(real_time) <= 2022));
time         = time((year(time) >= 1984) & (year(time) <= 2019));
real_time    = real_time((year(real_time) >= 2017) & (year(real_time) <= 2022));
GDPsolera    = NaN(length(time), length(real_time));
for i_month = 1:length(real_time)
    GDPsolera(:, i_month) = state_median(signal_index(i_month), :, i_month);
end

% Fix GDPplus missing month (Jan-2019)
i_Jan2019 = find(year(real_time) == 2019 & month(real_time) == 1);
GDPplus   = [GDPplus(:, 1:(i_Jan2019-1)), NaN(length(time), 1), GDPplus(:, i_Jan2019:end)];

% Extract dimensions
n_month   = 72;
n_quarter = 144;
Gsolera   = zeros(n_month, n_quarter);
Gplus     = zeros(n_month, n_quarter);
Go        = zeros(n_month, n_quarter);

% Start at 2015Q1 end at 2020Q4
init_quarter  = 133; %2017Q1
final_quarter = 144; %2019Q4
init_month    = 4;   %April 2015
for i_quarter = init_quarter:final_quarter
    for i_month = init_month:n_month
        Gsolera(i_month, i_quarter) = squeeze(state_median(signal_index(i_month), i_quarter, i_month));
        Gplus(i_month, i_quarter)   = GDPplus(i_quarter, i_month);
        Go(i_month, i_quarter)      = GDO(i_quarter, i_month);
    end
end

% Create figure
fig0   = figure;
set(fig0, 'units', 'normalized', 'position', [0 0 1 1])

for ifigure = 1:(final_quarter-init_quarter+1)

    % Create plot
    quarter    = ifigure+init_quarter-1;
    firstmonth = ifigure*3+1;
    subplot(3, 4, ifigure)
    h = plot(real_time(firstmonth:n_month), Gsolera(firstmonth:end, quarter), ...
        real_time(firstmonth:n_month), Gplus(firstmonth:end, quarter), ...
        real_time(firstmonth:n_month), Go(firstmonth:end, quarter));
    title(datestr(time(quarter), 'yyyy-QQ'))

    % Tune axis
    xlim([real_time(1) real_time(end)])
    xticks([real_time(7) real_time(19) real_time(31) real_time(43) real_time(55) real_time(67)])
    xticklabels({'2017', '2018', '2019', '2020', '2021', '2022'})
    set(gco,'fontsize', 12);
    set(gca,'fontsize', 12);
    ylim([-2.5 4.5])
    set(h, 'linewidth', 3, {'linestyle'}, {'-'; '-.'; ':'})
    markers = {'+'; 'x'; 's'};
    set(h, {'marker'}, markers, 'markersize', 4)
    set(h, {'color'}, {dark_green; grey; orange});
    set(gca, 'fontsize', 12)
    line([real_time(1) real_time(end)], [0 0], 'color', [0.6 0.6 0.6], 'linestyle', '-', 'linewidth', 0.5)
    for yr = 1:5
        line([datetime(2017+yr, 1, 1) datetime(2017+yr, 1, 1)], [-2.5 4.5], 'color', [0.6 0.6 0.6], 'linewidth', 0.5, 'linestyle', '-')
    end

end

% Save figure
print(fig0, [fig_dir 'stability_revisions.eps'], '-depsc')
