%%% REAL TIME POST %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Clear memory
clear
clc
close all

% Set filename
fname = '';

% Add paths and directories
addpath(['..' filesep 'simulation' filesep 'v0' filesep 'functions'])
data_dir = ['..' filesep 'data' filesep 'output' filesep];
mat_dir  = ['output' filesep 'matfiles' filesep];

% Set code parameters
n_MC = 1e4;

% Load and arrange data
datasets  = load([data_dir 'data.mat']);
time      = datetime(datenum(datasets.GDE.time, 'yyyyQQ'), 'convertfrom', 'datenum');
subsample = (datetime(1984, 1, 1) <= time) & (time < datetime(2023, 1, 1));
time      = time(subsample, :);
Y_GDE     = 100*log( [datasets.GDE.early(subsample, :), ...
                      datasets.GDE.late(subsample, :), ...
                      datasets.GDE.compr(subsample, :)] )';
Y_GDI     = 100*log( [datasets.GDI.early(subsample, :), ...
                      datasets.GDI.late(subsample, :), ...
                      datasets.GDI.compr(subsample, :)] )';
Y         = [Y_GDE; Y_GDI];

% Compute an store dimensions in a vector
N_c  = length(datasets.GDE.compr_aux);
N_E  = size(Y_GDE, 1);
N_I  = size(Y_GDI, 1);
T    = nnz(subsample);
dims = [N_c, N_E, N_I, T];
                   
% Define vintage indicators    
date_vintage = vintage2date( [datasets.GDE.early_vintage(subsample, :), ...
                              datasets.GDE.late_vintage(subsample, :), ...
                              datasets.GDE.compr_vintage(subsample, :), ...
                              datasets.GDI.early_vintage(subsample, :), ...
                              datasets.GDI.late_vintage(subsample, :), ...
                              datasets.GDI.compr_vintage(subsample, :)]' );

% Define signal indicators
delta_signal = NaN(N_E+N_I, T);
date_signal  = vintage2date( datasets.GDE.compr_aux );
for i_c = 1:(N_c-1)
    delta_signal(date_vintage >= date_signal(i_c) & date_vintage < date_signal(i_c+1)) = i_c;
end
delta_signal(date_vintage >= date_signal(N_c)) = N_c;
delta_signal = fillmissing(delta_signal, 'next', 2);     % impute missing entries of old signals using next available
delta_signal = fillmissing(delta_signal, 'previous', 2); % impute missing entries of new/discontinued signals using previous available

% Store indicators in 3D array
delta = zeros(N_E+N_I, N_c, T);
for i_c = 1:N_c
    delta(:, i_c, :) = double(delta_signal == i_c);
end

% Load parameter estimates
estimates = load([mat_dir sprintf('estimates_real_time%s.mat', fname)]);


%% WEIGHT CALCULATION

% Set prior
prior    = construct_prior(dims);
mu       = mean(Y(:, 1), 'omitnan')*ones(N_c, 1);
prior.mu = mu;

% Set numerical parameters for real-time exercise
target_year_grid    = [2001, 2008, 2019, 2014];
target_quarter_grid = [   1,    4,    2,    3]; 
base_year_grid      = [2002, 2009, 2019, 2014];
base_month_grid     = [   9,    1,    7,   10];
n_month_grid        = [  24,   24,   36,   60];
n_periods           = length(base_year_grid);
n_worker            = max(n_month_grid);

for i_period = 1:n_periods

% Extract dates
target_year    = target_year_grid(i_period);
target_quarter = target_quarter_grid(i_period);
base_year      = base_year_grid(i_period);
base_month     = base_month_grid(i_period);
n_month        = n_month_grid(i_period); 
real_time      = datetime(base_year, base_month-1+(1:n_month)', 1);

% Preallocate parameters, states, forecasts and weights
n_perc            = 99;
grid_perc         = linspace(1/(n_perc+1), n_perc/(n_perc+1), n_perc);
state_save        = NaN(N_c, T, n_perc, n_month);
state_mean        = NaN(N_c, T, n_month);
forecast_mean     = NaN(N_E+N_I, T, n_month);
weight_level_mean = NaN(N_c, N_E+N_I, T, n_month);
weight_diff_mean  = NaN(N_c, N_E+N_I, T, n_month);

% Construct state-space model with median parameters
t_weight = (year(time) == target_year & quarter(time) == target_quarter);
param    = estimates.param_save(:, 2, find(estimates.real_time >= datetime(target_year, 3*target_quarter, 1), 1, 'first'));
SSM      = construct_SSM(param, delta, dims, mu);
n_state  = size(SSM.F, 1);

% Run real-time exercise          
par_pool = parpool(n_worker);
parfor i_month = 1:n_month

    % Set seed for random number generator
    rng(1914)
        
    % Construct data with available measurements
    Y_tmp       = Y;
    ref_vintage = real_time(i_month);
    mat_vintage = (date_vintage <= ref_vintage);    
    Y_vintage   = NaN(N_E+N_I, T);
    Y_vintage(mat_vintage) = Y_tmp(mat_vintage);
    Z_vintage   = NaN(N_E+N_I, T);
    Z_vintage(mat_vintage) = 0;
    [~, state_tmp_0]       = fast_smoother(Z_vintage, SSM);

    % Run simulation smoother
    states  = NaN(n_state, T, n_MC);
    for i_MC = 1:n_MC
        states(:, :, i_MC) = simulation_smoother(Y_vintage, SSM);
    end
    dstates = 4*(states(1:N_c, :, :) - states(N_c + (1:N_c), :, :));
    state_i = quantile(dstates, grid_perc, 3);

    % Run fast smoother
    [~, state_tmp] = fast_smoother(Y_vintage, SSM);
    smooth_i       = 4*(state_tmp(1:N_c, :) - state_tmp(N_c + (1:N_c), :));
    
    % Compute forecasts
    forecast_i = NaN(N_E+N_I, T);
    for t = 1:T
        forecast_i(:, t) = SSM.H(:, :, t)*state_tmp(:, t);
    end
    
    % Compute weights
    weight_level = NaN(N_c, N_E+N_I, T);
    weight_diff  = NaN(N_c, N_E+N_I, T);
    for i = 1:(N_E+N_I)
        for t = 1:T
            if ~isnan(Z_vintage(i, t))
                Z_tmp          = Z_vintage;
                Z_tmp(i, t)    = 1;
                [~, state_tmp] = fast_smoother(Z_tmp, SSM);
                state_tmp      = state_tmp - state_tmp_0;
                weight_level(:, i, t) = state_tmp(1:N_c, t_weight);
                weight_diff(:, i, t)  = 4*(state_tmp(1:N_c, t_weight) - state_tmp(N_c+(1:N_c), t_weight));
            end
        end
    end
    
    % Save output
    parsave([mat_dir sprintf('results_%d.mat', i_month)], state_i, smooth_i, forecast_i, weight_level, weight_diff);
    
end

% Store output
for i_month = 1:n_month
    results = load([mat_dir sprintf('results_%d.mat', i_month)]);
    state_save(:, :, :, i_month)        = results.states;
    state_mean(:, :, i_month)           = results.smoothed;
    forecast_mean(:, :, i_month)        = results.forecast;
    weight_level_mean(:, :, :, i_month) = results.weight_level;
    weight_diff_mean(:, :, :, i_month)  = results.weight_diff;
    delete([mat_dir sprintf('results_%d.mat', i_month)])
end
results                   = struct();
results.time              = time;
results.real_time         = real_time;
results.date_vintage      = date_vintage;
results.Y                 = Y;
results.dims              = dims;
results.delta             = delta;
results.prior             = prior;
results.param             = param;
results.state_save        = state_save;
results.state_mean        = state_mean;
results.forecast_mean     = forecast_mean;
results.weight_level_mean = weight_level_mean;
results.weight_diff_mean  = weight_diff_mean;
save([mat_dir sprintf('estimates_real_time_%dQ%d%s.mat', ...
    target_year, target_quarter, fname)], '-struct', 'results')
delete(par_pool)

end


%% LOCAL FUNCTIONS

% Save results during parallel loop
function parsave(fname, states, smoothed, forecast, weight_level, weight_diff)
    res              = struct();
    res.states       = states;
    res.smoothed     = smoothed;
    res.forecast     = forecast;
    res.weight_level = weight_level;
    res.weight_diff  = weight_diff;
    save(fname, '-struct', 'res')
end