function SSM = construct_SSM(param, delta, dims, mu)
% CONSTRUCT_SSM  Construct state-space form of GDP solera model:
%   Y_(Et)     = delta_(Et) * x_t + v_(Et),
%   Y_(It)     = delta_(It) * x_t + v_(It),
%   Dx_(t+1)   = mu_X + rho_X .* Dx_t + chol(Sigma_X) .* eps_(Xt),
%   v_(E, t+1) = rho_E .* v_(Et) + chol(Sigma_E) .* eps_(Et),
%   v_(I, t+1) = rho_I .* v_(It) + chol(Sigma_I) .* eps_(It).
%
%   Initial condition is distributed as
%   (x_1, v_(E1), v_(I1)) ~ N(mu_1, Sigma_1).
%
%   Shocks and common factors are distributed as
%   eps_(Xt) ~ N(0_(N_cx1), I_(N_c)),
%   eps_(Et) ~ N(0_(N_Ex1), I_(N_E)),
%   eps_(It) ~ N(0_(N_Ix1), I_(N_I)).
%
%   SSM = CONSTRUCT_SSM(PARAM, DELTA, DIMS, MU, GAM) returns state-space
%   model matrices based on parameters PARAM, signal indicators DELTA,
%   dimensions DIMS, initial mean of signals MU and correlation of
%   measurement errors GAM:
%     PARAM is (2*N_c + N_c^2 + N_E + N_E^2 + N_I + N_I^2)x1, see vec_parameters.m.
%     DELTA is NxN_c (w/N=N_E+N_I), contains signal indicators delta_(Et) 
%     and delta_(It).
%     DIMS = [N_c, N_E, N_I, T], MU is N_cx1 and GAM is scalar
%     SSM is struct:
%     - SSM.H is NxMxT (w/M=2*N_c+N_E+N_I), time varying as delta.
%     - SSM.C is Mx1.
%     - SSM.F is MxM.
%     - SSM.G is MxK (w/K=N_c+N_E+N_I).
%     - SSM.mu_1 is Mx1.
%     - SSM.Sigma_1 is MxM.
%
%   Version: 2023 Jul 12 - Tincho Almuzara - Matlab R2017b

% Recover dimensions
N_c = dims(1);
N_E = dims(2);
N_I = dims(3);
T   = dims(4);
N   = N_E + N_I;

% Control correlation between common components of signal and noises
if ((nargin < 4) || isempty(mu)),  mu = zeros(N_c, 1); end; mu(isnan(mu)) = 0;

% Extract parameters from vector
[mu_X, rho_X, Sigma_X, ...
       rho_E, Sigma_E, ...
       rho_I, Sigma_I] = map_parameter(param, dims);
            
% Form state space matrices
H = NaN(N, 2*N_c+N, T);
for t = 1:T 
    H(:, :, t) = [delta(:, :, t), zeros(N, N_c), ...
                  blkdiag(eye(N_E), eye(N_I))];
end
Sigma_eps = (1e-4)*eye(N_E+N_I);
C = [mu_X; zeros(N_c+N, 1)];
F = blkdiag([diag(rho_X)+eye(N_c), -diag(rho_X); ...
             eye(N_c),             zeros(N_c)], ...
            diag(rho_E), diag(rho_I));
G = blkdiag([eye(N_c); zeros(N_c)], ...
             eye(N_E), ...
             eye(N_I));
Sigma_eta = blkdiag(Sigma_X, Sigma_E, Sigma_I);
Sigma_1   = zeros(2*N_c+N);
for t = 1:T, Sigma_1 = F*Sigma_1*(F') + G*(G'); end

% Pass output as struct
SSM           = struct();
SSM.H         = H;
SSM.Sigma_eps = Sigma_eps;
SSM.C         = C;
SSM.F         = F;
SSM.G         = G;
SSM.Sigma_eta = Sigma_eta;
SSM.mu_1      = [repmat(mu(:), [2, 1]); zeros(N, 1)];
SSM.Sigma_1   = Sigma_1; 

end