function prior = construct_prior(dims)
% CONSTRUCT_PRIOR  Construct prior for parameters of GDP solera model:
%   mu_X, rho_X | Sigma_X ~ N(m_X, kron(Pi_X, Sigma_X)),
%   Sigma_X               ~ W^(-1)(nu_X*S_X, nu_X),
%   rho_E | Sigma_E       ~ N(m_E, kron(Pi_E, Sigma_E)),
%   Sigma_E               ~ W^(-1)(nu_E*S_E, nu_E),
%   rho_I | Sigma_I       ~ N(m_I, kron(Pi_I, Sigma_I)),
%   Sigma_I               ~ W^(-1)(nu_I*S_I, nu_I).
% 
%   PRIOR = CONSTRUCT_PRIOR(DIMS) returns prior based on dimensions DIMS:
%     DIMS = [N_C, N_E, N_I, T]:
%     - N_C is number of underlying comprehensive measures.
%     - N_E is number of GDP measurements.
%     - N_I is number of GDI measurements.
%     - T is the time series dimension.
%     PRIOR is struct:
%     - PRIOR.P_X is 2x2, precision of (mu_X, rho_X).
%     - PRIOR.m_X is 2N_Cx1 mean of (mu_X, rho_X).
%     - PRIOR.nu_X is degrees of freedom of Sigma_X.
%     - PRIOR.S_X  is N_CxN_C (approx) mean of Sigma_X.
%     - PRIOR.P_E is scalar, precision of rho_E.
%     - PRIOR.m_E is N_Ex1 mean of rho_E.
%     - PRIOR.nu_E is degrees of freedom of Sigma_E.
%     - PRIOR.S_E is N_ExN_I (approx) mean of Sigma_E.
%     - PRIOR.P_I is scalar, precision of rho_I.
%     - PRIOR.m_I is N_Ix1 mean of rho_I.
%     - PRIOR.nu_I is degrees of freedom of Sigma_I.
%     - PRIOR.s2_I is N_IxN_I (approx) mean of Sigma_I.
% 
%   Version: 2023 Jul 12 - Tincho Almuzara - Matlab R2017b

% Recover dimensions
N_c = dims(1);
N_E = dims(2);
N_I = dims(3);

% Calibrate values from GDP-GDI paper
m_X = 0.37;
r_X = 0.50;
s_X = 0.48;
r_E = 0.00;
s_E = 0.10;
r_I = 0.80;
s_I = 0.18;

% Calibrate split between common and idiosyncratic variance
w_common_signal = 0.8;
w_common_error  = 0.2;

% Create prior struct
prior = struct();

% Parameters of GDP signal
prior.P_X  = blkdiag(10, 30);
prior.m_X  = [m_X*ones(N_c, 1); ...
              r_X*ones(N_c, 1)];
prior.nu_X = 20+N_c;
prior.S_X  = s_X*((1-w_common_signal)*eye(N_c) + w_common_signal*ones(N_c));

% Parameters of measurement error in GDE
prior.P_E  = 10;
prior.m_E  = r_E*ones(N_E, 1);
prior.nu_E = 20+N_E;
prior.S_E  = s_E*((1-w_common_error)*eye(N_E) + w_common_error*ones(N_E));

% Parameters of measurement error in GDI
prior.P_I  = 10;
prior.m_I  = r_I*ones(N_I, 1);
prior.nu_I = 20+N_I;
prior.S_I  = s_I*((1-w_common_error)*eye(N_I) + w_common_error*ones(N_I));

end