function L = log_posterior(param, states, prior)
% LOG_POSTERIOR Compute joint log-density of parameters (posterior). 
% 
%   L = LOG_POSTERIOR(PARAM, STATE, PRIOR) returns log-density value L for
%   a vector of parameters PARAM, state variables STATES and prior PRIOR. 
% 
%   Version: 2023 Jul 12 - Tincho Almuzara - Matlab R2017b

% Recover dimensions
N_c  = size(prior.s2_X, 1);
N_E  = size(prior.s2_E, 1);
N_I  = size(prior.s2_I, 1);
T    = size(states, 2);
dims = [N_c, N_E, N_I, T];

% Extract parameters
[mu_X, rho_X, lambda_X, sigma_X, ...
       rho_E, lambda_E, sigma_E, ...
       rho_I, lambda_I, sigma_I] = map_parameter(param, dims);

% Define function handle and options for inverse algorithm
symmetrize    = @(A) (A + A')/2;
option        = struct();
option.SYM    = true;
option.POSDEF = true;

% Preallocate log-density
L = 0;

%%% Compute log-density of parameters of signal
beta_X  = [mu_X; rho_X(:); lambda_X];
sign_X  = sign(sum(lambda_X));
MatX    = eye(N_c) - (1/(1+N_c))*ones(N_c);
DX      = states(1:N_c, :) - states(N_c + (1:N_c), :);
fX      = states(2*N_c+1, :);
yx_X    = DX(:, 2:T)*([ones(1, T-1); DX(:, 1:(T-1)); fX(2:T)]');
xx_X    = [ones(1, T-1); DX(:, 1:(T-1)); fX(2:T)]*([ones(1, T-1); DX(:, 1:(T-1)); fX(2:T)]');
P_X_inv = symmetrize( linsolve(symmetrize( kron(prior.P_X, MatX) + kron(xx_X, eye(N_c)) ), eye((2+N_c)*N_c), option) );
m_X     = P_X_inv*(kron(prior.P_X, MatX)*prior.m_X + yx_X(:));
e_X     = NaN(N_c, T-1);
for t = 1:(T-1), e_X(:, t) = DX(:, t+1) - mu_X - rho_X*DX(:, t) - lambda_X*sign_X*fX(t+1); end
nu_X    = prior.nu_X + T-1;
s2_X    = (prior.nu_X/nu_X)*prior.s2_X + (1/nu_X)*sum(e_X.^2, 2);
L       = L + sum( gamlogpdf(1./(sigma_X.^2), nu_X/2, 2./(nu_X*s2_X)) ); 
L       = L + mvnlogpdf(beta_X, m_X, symmetrize( kron(eye(2+N_c), diag(sigma_X))*P_X_inv*kron(eye(2+N_c), diag(sigma_X)) ));

%%% Compute log-density of parameters of measurement error in GDE
beta_E  = [rho_E; lambda_E]; 
sign_E  = sign(sum(lambda_E));
MatE    = eye(N_E) - (0.25/(1+0.25*N_E))*ones(N_E);
vE      = states((2*N_c+1) + (1:N_E), :);
fE      = states(2*N_c+1+N_E+1, :);
xy_E    = [sum(vE(:, 2:T).*vE(:, 1:(T-1)), 2); ...
           sum(vE(:, 2:T).*fE(2:T), 2)];    
xx_E    = [diag( sum(vE(:, 1:(T-1)).^2, 2) ),       diag( sum(vE(:, 1:(T-1)).*fE(2:T), 2) ); ...
           diag( sum(vE(:, 1:(T-1)).*fE(2:T), 2) ), sum(fE(2:T).^2)*eye(N_E)];
P_E_inv = symmetrize( linsolve(symmetrize(kron(prior.P_E, MatE) + xx_E), eye(2*N_E), option) );
m_E     = P_E_inv*(kron(prior.P_E, MatE)*prior.m_E + xy_E);
e_E     = NaN(N_E, T-1);
for t = 1:(T-1), e_E(:, t) = vE(:, t+1) - rho_E.*vE(:, t) - lambda_E*sign_E*fE(t+1); end
nu_E    = prior.nu_E + T-1;
s2_E    = (prior.nu_E/nu_E)*prior.s2_E + (1/nu_E)*sum(e_E.^2, 2);
L       = L + sum( gamlogpdf(1./(sigma_E.^2), nu_E/2, 2./(nu_E*s2_E)) );
L       = L + mvnlogpdf(beta_E, m_E, symmetrize( kron(eye(2), diag(sigma_E))*P_E_inv*kron(eye(2), diag(sigma_E)) ));

%%% Compute log-density of parameters of measurement error in GDI
beta_I  = [rho_I; lambda_I]; 
sign_I  = sign(sum(lambda_I));
MatI    = eye(N_I) - (0.25/(1+0.25*N_I))*ones(N_I);
vI      = states((2*N_c+1+N_E+1) + (1:N_I), :);
fI      = states(2*N_c+1+N_E+1+N_I+1, :);
xy_I    = [sum(vI(:, 2:T).*vI(:, 1:(T-1)), 2); ...
           sum(vI(:, 2:T).*fI(2:T), 2)];
xx_I    = [diag( sum(vI(:, 1:(T-1)).^2, 2) ),       diag( sum(vI(:, 1:(T-1)).*fI(2:T), 2) ); ...
           diag( sum(vI(:, 1:(T-1)).*fI(2:T), 2) ), sum(fI(2:T).^2)*eye(N_I)];
P_I_inv = symmetrize( linsolve(symmetrize(kron(prior.P_I, MatI) + xx_I), eye(2*N_I), option) );
m_I     = P_I_inv*(kron(prior.P_I, MatI)*prior.m_I + xy_I);
e_I     = NaN(N_I, T-1);
for t = 1:(T-1), e_I(:, t) = vI(:, t+1) - rho_I.*vI(:, t) - lambda_I*sign_I*fI(t+1); end
nu_I    = prior.nu_I + T-1;
s2_I    = (prior.nu_I/nu_I)*prior.s2_I + (1/nu_I)*sum(e_I.^2, 2);
L       = L + sum( gamlogpdf(1./(sigma_I.^2), nu_I/2, 2./(nu_I*s2_I)) );
L       = L + mvnlogpdf(beta_I, m_I, symmetrize( kron(eye(2), diag(sigma_I))*P_I_inv*kron(eye(2), diag(sigma_I)) ));

end


%% LOCAL FUNCTIONS

function p = gamlogpdf(x, a, b)
    x = x(:);
    a = a(:);
    b = b(:);
    p = -gammaln(a)-a.*log(b)+(a-1).*log(x)-x./b;
end

function p = mvnlogpdf(x, mu, Sigma)
    x  = x(:);
    mu = mu(:);
    n  = size(x(:), 1);
    p  = -(n/2)*log(2*pi) - (1/2)*log(det(Sigma)) - (x-mu)'*(Sigma\(x-mu))/2;
end